@@ -770,11 +770,7 @@ next_context:
 
 ex function::expand(unsigned options) const
 {
-	// Only expand arguments when asked to do so
-	if (options & expand_options::expand_function_args)
-		return inherited::expand(options);
-	else
-		return (options == 0) ? setflag(status_flags::expanded) : *this;
+	return inherited::expand(options);
 }
 
 ex function::eval(int level) const
@@ -913,9 +909,9 @@ ex function::evalf(int level, PyObject* kwds) const
 	throw(std::logic_error("function::evalf(): invalid nparams"));
 }
 
-unsigned function::calchash() const
+long function::calchash() const
 {
-	unsigned v = golden_ratio_hash(golden_ratio_hash((p_int)tinfo()) ^ serial);
+	long v = golden_ratio_hash(golden_ratio_hash((p_int)tinfo()) ^ serial);
 	for (size_t i=0; i<nops(); i++) {
 		v = rotate_left(v);
 		v ^= this->op(i).gethash();
