@@ -52,6 +52,7 @@
 #include "numeric.h"
 #include "operators.h"
 #include "power.h"
+#include "function.h"
 #include "archive.h"
 #include "tostring.h"
 #include "utils.h"
@@ -485,7 +486,7 @@ numeric::numeric(PyObject* o, bool force_py) : basic(&numeric::tinfo_static) {
                 if (PyInt_Check(o)) {
                         t = MPZ;
                         mpz_init(v._bigint);
-                        mpz_set_si(v._bigint, PyInt_AS_LONG(o));
+                        mpz_set_si(v._bigint, PyInt_AsLong(o));
                         hash = _mpz_pythonhash(v._bigint);
                         setflag(status_flags::evaluated | status_flags::expanded);
                         Py_DECREF(o);
@@ -1084,8 +1085,8 @@ const numeric numeric::div(const numeric &other) const {
                         if (PyInt_Check(v._pyobject)) {
                                 if (PyInt_Check(other.v._pyobject)) {
                                         // This branch happens at startup.
-                                        PyObject* o = PyNumber_TrueDivide(Integer(PyInt_AS_LONG(v._pyobject)),
-                                                Integer(PyInt_AS_LONG(other.v._pyobject)));
+                                        PyObject* o = PyNumber_TrueDivide(Integer(PyInt_AsLong(v._pyobject)),
+                                                Integer(PyInt_AsLong(other.v._pyobject)));
                                         // I don't 100% understand why I have to incref this, 
                                         // but if I don't, Sage crashes on exit.
                                         Py_INCREF(o);
@@ -2123,6 +2124,72 @@ const numeric numeric::bernoulli() const {
         PY_RETURN(py_funcs.py_bernoulli);
 }
 
+static PyObject* py_tuple_from_numvector(const std::vector<numeric>& vec)
+{
+        PyObject* list = PyTuple_New(vec.size());
+        if (!list)
+                throw(std::runtime_error("py_list_from_numvector(): PyList_New returned NULL"));
+        int pos = 0;
+        for (const numeric& num : vec) {
+                PyObject *numobj = num.to_pyobject();
+                int ret = PyTuple_SetItem(list, pos++, numobj);
+                if (ret)
+                        throw(std::runtime_error("py_list_from_numvector(): PyList_Append unsuccessful"));
+        }
+        return list;
+}
+
+const numeric numeric::hypergeometric_pFq(const std::vector<numeric>& a, const std::vector<numeric>& b, PyObject *parent) const {
+        PyObject *lista = py_tuple_from_numvector(a);
+        PyObject *listb = py_tuple_from_numvector(b);
+        PyObject *z = to_pyobject();
+
+        // call opt.evalf_f with this tuple
+        PyObject* m = PyImport_ImportModule("sage.functions.hypergeometric");
+        if (!m)
+                py_error("Error importing hypergeometric");
+        PyObject* hypfunc = PyObject_GetAttrString(m, "hypergeometric");
+        if (!hypfunc)
+                py_error("Error getting hypergeometric attribute");
+        PyObject *itemp, *itema;
+        if (parent and PyDict_CheckExact(parent)) {
+                itemp = PyDict_GetItemString(parent, const_cast<char*>("parent"));
+                itema = PyDict_GetItemString(parent, const_cast<char*>("algorithm"));
+        }
+        else {
+                itemp = parent;
+                if (itemp)
+                        Py_INCREF(itemp);
+                itema = NULL;
+        }
+        if (itemp == NULL) {
+                itemp = RR;
+                Py_INCREF(itemp);
+        }
+        PyObject* name = PyString_FromString(const_cast<char*>("_evalf_"));
+        PyObject* pyresult = PyObject_CallMethodObjArgs(hypfunc, name, lista, listb, z, itemp, itema, NULL);
+        Py_DECREF(m);
+        Py_DECREF(name);
+        if (itemp)
+                Py_DECREF(itemp);
+        if (itema)
+                Py_DECREF(itema);
+        Py_DECREF(hypfunc);
+        if (!pyresult) {
+                throw(std::runtime_error("numeric::hypergeometric_pFq(): python function hypergeometric::_evalf_ raised exception"));
+        }
+        if ( pyresult == Py_None ) {
+                throw(std::runtime_error("numeric::hypergeometric_pFq(): python function hypergeometric::_evalf_ returned None"));
+        }
+        // convert output Expression to an ex
+        ex eval_result = py_funcs.pyExpression_to_ex(pyresult);
+        Py_DECREF(pyresult);
+        if (PyErr_Occurred()) {
+                throw(std::runtime_error("numeric::hypergeometric_pFq(): python function (Expression_to_ex) raised exception"));
+        }
+        return ex_to<numeric>(eval_result);
+}
+
 const numeric numeric::isqrt() const {
         PY_RETURN(py_funcs.py_isqrt);
 }
@@ -2549,6 +2616,10 @@ const numeric bernoulli(const numeric &n) {
         return n.bernoulli();
 }
 
+const numeric hypergeometric_pFq(const std::vector<numeric>& a, const std::vector<numeric>& b, const numeric &z, PyObject* parent) {
+        return z.hypergeometric_pFq(a, b, parent);
+}
+
 /** Fibonacci number.  The nth Fibonacci number F(n) is defined by the
  *  recurrence formula F(n)==F(n-1)+F(n-2) with F(0)==0 and F(1)==1.
  *
