@@ -561,6 +561,7 @@ ex power::eval(int level) const
 	if (is_exactly_a<power>(ebasis) && ebasis.op(0).info(info_flags::positive) && ebasis.op(1).info(info_flags::real))
 		return power(ebasis.op(0), ebasis.op(1) * eexponent);
 
+
 	if (exponent_is_numerical) {
 		// ^(c1,c2) -> c1^c2  (c1, c2 numeric(),
 		// except if c1,c2 are rational, but c1^c2 is not)
@@ -663,7 +664,7 @@ ex power::eval(int level) const
                                                         lead_coeff()).div(icont);
 
                                 const bool canonicalizable = lead_coeff.is_integer();
-                                const bool unit_normal = lead_coeff.is_pos_integer();
+                                const bool unit_normal = lead_coeff.is_pos_integer() || lead_coeff.is_parent_pos_char();
                                 if (canonicalizable && (! unit_normal))
                                         icont = icont.mul(*_num_1_p);
 
@@ -727,6 +728,18 @@ ex power::eval(int level) const
 			return ncmul(exvector(num_exponent.to_int(), ebasis), true);
 		}
 	}
+
+	// Reduce x^(c/log(x)) to exp(c) if x is positive
+	if (ebasis.info(info_flags::positive)) {
+		if (eexponent.is_equal(1/log(ebasis)))
+			return exp(log(basis)*exponent);
+		else if (is_exactly_a<mul>(eexponent)) {
+			for (size_t i=0; i < eexponent.nops(); i++) {
+				if (eexponent.op(i).is_equal(1/log(ebasis)))
+					return exp(log(basis)*exponent);
+			}
+		}
+	}
 	
 	if (are_ex_trivially_equal(ebasis,basis) &&
 	    are_ex_trivially_equal(eexponent,exponent)) {
